/*
 * 쐬: 2003/12/07
 *
 */
package com.muimi.pt;

import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorDescriptor;
import org.eclipse.ui.IFileEditorMapping;
import org.eclipse.ui.IPerspectiveDescriptor;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.WorkbenchPlugin;
import org.eclipse.ui.internal.registry.IViewDescriptor;
import org.eclipse.ui.part.ViewPart;


/**
 * @author miya
 *
 */
public class PTViewPart extends ViewPart
{


	private TreeViewer treeViewer;


	public void createPartControl(Composite parent) {

		treeViewer = new TreeViewer(parent);

		treeViewer.setContentProvider(new WSTreeContentProvider());

		treeViewer.setLabelProvider(new WSTreeLabelProvider());

		treeViewer.setInput("ROOT");
		
		treeViewer.addDoubleClickListener(new WSTreeDoubleClickListener());
		
		getSite().setSelectionProvider(treeViewer);
	}


	public void setFocus() {
	}



	class WSTreeContentProvider implements ITreeContentProvider{

		public Object[] getChildren(Object parent) {
			//IAdaptablełȂ΃vLVʂ
			Object[] childs = getChildrenInternal(parent);
			childs = AdapterProxy.newArrayInstance(childs);
			return childs;
		}

		public Object[] getChildrenInternal(Object parent) {
			if("ROOT".equals(parent)){
				return new String[]{
					"View",
					"Editor",
					"Perspective",
					//"ActionSet",//next time
				};
			}else if("View".equals(parent)){

				return WorkbenchPlugin.getDefault().getViewRegistry().getViews();

			}else if("Editor".equals(parent)){

				IFileEditorMapping[] maps = PlatformUI.getWorkbench().getEditorRegistry().getFileEditorMappings();				
				PlatformUI.getWorkbench().getEditorRegistry().getFileEditorMappings();
				Set set = new HashSet();
				for(int i=0; i<maps.length; i++){
					IFileEditorMapping map = maps[i];
					IEditorDescriptor[] descs = map.getEditors();
					set.addAll(Arrays.asList(descs));
				}
				return set.toArray(new IEditorDescriptor[0]);

			}else if("Perspective".equals(parent)){
				
				return PlatformUI.getWorkbench().getPerspectiveRegistry().getPerspectives();

			}
//			else if("ActionSet".equals(parent)){
//
//				return WorkbenchPlugin.getDefault().getActionSetRegistry().getActionSets();
//			}
						
			return new String[0];
		}

		public Object getParent(Object element) {
			return null;
		}

		public boolean hasChildren(Object element) {
			return getChildren(element).length > 0;
		}

		public Object[] getElements(Object input) {
			return getChildren(input);
		}

		public void dispose() {
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
	}



	class WSTreeLabelProvider extends LabelProvider{

		public String getText(Object element) {

			try {
				String label = null;
				{
					Method method = element.getClass().getMethod("getLabel", new Class[0]);
					if(method != null && method.getReturnType().equals(String.class)){
						label =  (String)method.invoke(element, new Object[0]);
					}
				}
				
				String id = null;
				{
					Method method = element.getClass().getMethod("getId", new Class[0]);
					if(method != null && method.getReturnType().equals(String.class)){
						id=  (String)method.invoke(element, new Object[0]);
					}
				}
				
				if(label != null){
					return label + "(" + id + ")";
				}

			} catch (Exception ignore) {
			}

			return super.getText(element);
		}


		public Image getImage(Object element) {
			
			if(element instanceof String){
				return PlatformUI.getWorkbench().getSharedImages().getImage(ISharedImages.IMG_OBJ_FOLDER);
			}
			
			try {
				Method method = element.getClass().getMethod("getImageDescriptor", new Class[0]);
				if(method != null && method.getReturnType().equals(ImageDescriptor.class)){
					ImageDescriptor desc = (ImageDescriptor)method.invoke(element, new Object[0]);
					return desc.createImage();
				}
			} catch (Exception ignore) {
			}

			return super.getImage(element);
		}

	}

	class WSTreeDoubleClickListener implements IDoubleClickListener{

		public void doubleClick(DoubleClickEvent event) {
			IStructuredSelection ss = (IStructuredSelection)event.getSelection();
			Object element = ss.getFirstElement();
			try {
				doubleClickInternal(element);
			} catch (Exception e) {
				PTPlugin.log("unexpected error", e);
			}
		}
		
		private void doubleClickInternal(Object element) throws Exception{
			if(element instanceof IViewDescriptor){
				IViewDescriptor desc = (IViewDescriptor)element;
				PTPlugin.getActivePage().showView(desc.getId());

				
			}else if(element instanceof IPerspectiveDescriptor){
				IPerspectiveDescriptor desc = (IPerspectiveDescriptor)element;
				PlatformUI.getWorkbench().showPerspective(
					desc.getId(), 
					PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				);

			}else{
				PTPlugin.debug("double clicked. but i don't nothing about " + element);
			}
		}
		
	}



}
